@@ -75,7 +75,6 @@ PA_MODULE_USAGE(
         "rate=<sample rate> "
         "channels=<number of channels> "
         "path=<device object path> "
-        "auto_connect=<automatically connect?> "
         "sco_sink=<SCO over PCM sink name> "
         "sco_source=<SCO over PCM source name>");
 
@@ -94,7 +93,6 @@ static const char* const valid_modargs[] = {
     "rate",
     "channels",
     "path",
-    "auto_connect",
     "sco_sink",
     "sco_source",
     NULL
@@ -148,7 +146,6 @@ struct userdata {
     pa_hook_slot *transport_speaker_changed_slot;
 
     pa_bluez4_discovery *discovery;
-    bool auto_connect;
 
     char *output_port_name;
     char *input_port_name;
@@ -2417,7 +2414,7 @@ static pa_hook_result_t discovery_hook_cb(pa_bluez4_discovery *y, const pa_bluez
     else
         return PA_HOOK_OK;
 
-    pa_module_unload(u->core, u->module, true);
+    pa_module_unload(u->module, true);
 
     return PA_HOOK_OK;
 }
@@ -2460,12 +2457,6 @@ int pa__init(pa_module *m) {
         goto fail;
     }
 
-    u->auto_connect = true;
-    if (pa_modargs_get_value_boolean(ma, "auto_connect", &u->auto_connect)) {
-        pa_log("Failed to parse auto_connect= argument");
-        goto fail;
-    }
-
     channels = u->sample_spec.channels;
     if (pa_modargs_get_value_u32(ma, "channels", &channels) < 0 ||
         !pa_channels_valid(channels)) {
