@@ -17,9 +17,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public
-  License along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  License along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #include <sys/types.h>
@@ -37,7 +35,6 @@
 
 typedef struct pa_memblockq pa_memblockq;
 
-
 /* Parameters:
 
    - name:      name for debugging purposes
@@ -88,7 +85,7 @@ int pa_memblockq_push(pa_memblockq* bq, const pa_memchunk *chunk);
 int pa_memblockq_push_align(pa_memblockq* bq, const pa_memchunk *chunk);
 
 /* Manipulate the write pointer */
-void pa_memblockq_seek(pa_memblockq *bq, int64_t offset, pa_seek_mode_t seek, pa_bool_t account);
+void pa_memblockq_seek(pa_memblockq *bq, int64_t offset, pa_seek_mode_t seek, bool account);
 
 /* Return a copy of the next memory chunk in the queue. It is not
  * removed from the queue. There are two reasons this function might
@@ -110,7 +107,7 @@ void pa_memblockq_drop(pa_memblockq *bq, size_t length);
 void pa_memblockq_rewind(pa_memblockq *bq, size_t length);
 
 /* Test if the pa_memblockq is currently readable, that is, more data than base */
-pa_bool_t pa_memblockq_is_readable(pa_memblockq *bq);
+bool pa_memblockq_is_readable(pa_memblockq *bq);
 
 /* Return the length of the queue in bytes */
 size_t pa_memblockq_get_length(pa_memblockq *bq);
@@ -126,7 +123,7 @@ size_t pa_memblockq_pop_missing(pa_memblockq *bq);
 int pa_memblockq_splice(pa_memblockq *bq, pa_memblockq *source);
 
 /* Set the queue to silence, set write index to read index */
-void pa_memblockq_flush_write(pa_memblockq *bq, pa_bool_t account);
+void pa_memblockq_flush_write(pa_memblockq *bq, bool account);
 
 /* Set the queue to silence, set write read index to write index*/
 void pa_memblockq_flush_read(pa_memblockq *bq);
@@ -179,13 +176,13 @@ void pa_memblockq_willneed(pa_memblockq *bq);
 /* Check whether the memblockq is completely empty, i.e. no data
  * neither left nor right of the read pointer, and hence no buffered
  * data for the future nor data in the backlog. */
-pa_bool_t pa_memblockq_is_empty(pa_memblockq *bq);
+bool pa_memblockq_is_empty(pa_memblockq *bq);
 
 /* Drop everything in the queue, but don't modify the indexes */
 void pa_memblockq_silence(pa_memblockq *bq);
 
 /* Check whether we currently are in prebuf state */
-pa_bool_t pa_memblockq_prebuf_active(pa_memblockq *bq);
+bool pa_memblockq_prebuf_active(pa_memblockq *bq);
 
 /* Return how many items are currently stored in the queue */
 unsigned pa_memblockq_get_nblocks(pa_memblockq *bq);
